plugins {
    java
    application

    idea

    id("com.diffplug.spotless") version "6.25.0"
}

group = "de.lul"
version = "1.0"

base {
    archivesName.set("cd-nmdp")
}

java {
    sourceCompatibility = JavaVersion.VERSION_17
    targetCompatibility = JavaVersion.VERSION_17

    withSourcesJar()
    withJavadocJar()
}

var defaultEncoding = "UTF-8"
tasks.withType<JavaCompile> { options.encoding = defaultEncoding }
tasks.withType<Javadoc> { options.encoding = defaultEncoding }
tasks.withType<Test> { systemProperty("file.encoding", "UTF-8") }

tasks.test {
    useJUnitPlatform()
    systemProperty("java.library.path", project.layout.buildDirectory.dir("lib").get().asFile)
}

idea {
    module {
        isDownloadJavadoc = true
        isDownloadSources = true
    }
}

repositories {
    mavenCentral()
}

spotless {
    java {
        palantirJavaFormat()
    }
    groovyGradle {
        greclipse()
    }
}

dependencies {
    implementation(project(":lib:parser"))

    // https://picocli.info/
    implementation("info.picocli", "picocli", "4.7.6")
    // https://github.com/google/guava
    implementation("com.google.guava", "guava", "33.2.1-jre")

    // https://tinylog.org/v2/
    implementation("org.tinylog:tinylog-impl:2.7.0")
    implementation("org.tinylog:slf4j-tinylog:2.7.0")
    // https://mvnrepository.com/artifact/org.ojalgo/ojalgo
    implementation("org.ojalgo:ojalgo:54.0.0")

    implementation("guru.nidi:graphviz-java:0.18.1")
}

application {
    mainClass.set("de.cmdp.Main")
}

