package de.cmdp;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

public final class Distribution<S> {
    private final Map<S, Double> distribution;

    private Distribution(Map<S, Double> distribution) {
        this.distribution = distribution;
    }

    public static <S> Distribution<S> of(Map<S, Double> map) {
        assert Math.abs(map.values().stream().mapToDouble(Double::doubleValue).sum() - 1.0) < 1e-10;
        return new Distribution<>(map);
    }

    public Set<S> support() {
        return distribution.keySet();
    }

    public Map<S, Double> map() {
        return distribution;
    }

    @Override
    public String toString() {
        return distribution.entrySet().stream()
                .map(e -> e.getKey() + "=>" + e.getValue())
                .collect(Collectors.joining(", ", "[", "]"));
    }

    public boolean isOnlySuccessor(S state) {
        return distribution.size() == 1 && distribution.containsKey(state);
    }
}
