package de.cmdp;

import java.util.HashMap;
import java.util.Map;
import java.util.stream.Collectors;

public record GlobalState(Map<ModuleGraph, ModuleGraph.ModuleState> states) {
    public GlobalState(Map<ModuleGraph, ModuleGraph.ModuleState> states) {
        this.states = Map.copyOf(states);
    }

    ModuleGraph.ModuleState moduleState(ModuleGraph module) {
        return states.get(module);
    }

    @Override
    public String toString() {
        Map<String, Object> variables = new HashMap<>();
        states.forEach((g, s) -> g.variables.forEach(v -> variables.put(v, s.value(v))));
        return variables.entrySet().stream()
                .sorted(Map.Entry.comparingByKey())
                .map(e -> e.getKey() + "=" + e.getValue())
                .collect(Collectors.joining(",", "[", "]"));
    }
}
