package de.cmdp;

import static guru.nidi.graphviz.model.Factory.mutNode;
import static guru.nidi.graphviz.model.Factory.to;

import com.google.common.collect.Iterables;
import guru.nidi.graphviz.attribute.Arrow;
import guru.nidi.graphviz.attribute.Label;
import guru.nidi.graphviz.attribute.Shape;
import guru.nidi.graphviz.attribute.Size;
import guru.nidi.graphviz.attribute.Style;
import guru.nidi.graphviz.engine.Format;
import guru.nidi.graphviz.engine.Graphviz;
import guru.nidi.graphviz.model.Factory;
import guru.nidi.graphviz.model.Link;
import guru.nidi.graphviz.model.MutableGraph;
import guru.nidi.graphviz.model.MutableNode;
import java.io.IOException;
import java.io.OutputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.ArrayDeque;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Queue;
import java.util.Set;
import java.util.function.Function;

public final class Render {
    private Render() {}

    public static void display(GlobalExplorer explorer, Set<GlobalState> initial, Function<GlobalState, String> label)
            throws IOException {
        MutableGraph g = Factory.mutGraph().setDirected(true);
        g.graphAttrs().add("pack", true);
        g.graphAttrs().add("packMode", "cluster");

        Queue<GlobalState> queue = new ArrayDeque<>(initial);
        Set<GlobalState> explored = new HashSet<>(initial);
        while (!queue.isEmpty()) {
            var state = queue.poll();

            for (GlobalAction action : explorer.actions(state)) {
                for (GlobalState globalState : action.support()) {
                    if (explored.add(globalState)) {
                        queue.add(globalState);
                    }
                }
            }
        }

        Map<GlobalState, Integer> index = new HashMap<>();
        explored.forEach(s -> index.put(s, index.size()));

        Map<GlobalState, MutableNode> nodeMap = new HashMap<>();
        explored.forEach(s -> nodeMap.put(
                s,
                mutNode("s_" + index.get(s))
                        .add(Label.of(label.apply(s)))
                        .add(Shape.BOX)
                        .add(Size.mode(Size.Mode.MINIMUM).margin(0, 0))));

        nodeMap.forEach((s, n) -> g.add(n));
        int actionCounter = 0;
        for (GlobalState state : explored) {
            for (GlobalAction action : explorer.actions(state)) {
                Label actionLabel = Label.of(action.label());
                if (action.support().size() == 1) {
                    var successor = Iterables.getOnlyElement(action.support());
                    nodeMap.get(state).addLink(to(nodeMap.get(successor)).add(actionLabel));
                } else {
                    MutableNode actionNode = mutNode("a_" + actionCounter)
                            .add(actionLabel)
                            .add(Size.mode(Size.Mode.MINIMUM).margin(0, 0))
                            .add(Shape.BOX)
                            .add(Style.DASHED);

                    nodeMap.get(state).addLink(to(actionNode).add(Style.DASHED).add(Arrow.NONE));
                    action.distribution().map().forEach((GlobalState t, Double p) -> {
                        Link target = to(nodeMap.get(t));
                        target.add(Label.of(String.format("%.2f", p)));
                        actionNode.addLink(target);
                    });
                }
                actionCounter += 1;
            }
        }
        try (OutputStream os = Files.newOutputStream(Path.of("out.png"))) {
            Graphviz.fromGraph(g).render(Format.PNG).toOutputStream(os);
        }
    }
}
